/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 					      Bologna Astronomical Observatory
 *----------------------------------------------------------------------------
 * File name 	: cxc_init.c
 * Author 	: Paolo Montegriffo
 * Created on	: May, 2002	
 * Language	: ANSI C
 * Description	: Initialize GTK interface for CataXcorr
 * Version      : 2.1.14 - May 05, 2003
 *--------------------------------------------------------------------------*/

#include "config.h"

#include <signal.h>
#ifdef HAVE_SYS_PARAM_H
#include <sys/param.h>
#endif
#ifdef HAVE_DIRENT_H
#include <dirent.h>
#endif

#ifdef G_OS_WIN32
#include <process.h>		/* For _getpid() */
#endif

#if HAVE_CONFIG_H
#  include "config.h"
#endif

#include "common.h"


#include <gtk/gtk.h>

#include "catalib.h"
#include "cpk_env.h"
#include "cpk_init.h"
#include "ccl_init.h"

/*----------------------------------------------------------------------------
 * 								Defines
 *--------------------------------------------------------------------------*/

#define LOGO_WIDTH_MIN  300
#define LOGO_HEIGHT_MIN 110
#define AUTHORS "Paolo Montegriffo"

#define SHOW_NEVER 0
#define SHOW_LATER 1
#define SHOW_NOW   2
#ifndef MAXPATHLEN
#define MAXPATHLEN 2048
#endif

/*----------------------------------------------------------------------------
 * 							Function prototypes
 *--------------------------------------------------------------------------*/



/*----------------------------------------------------------------------------
 * 							Global variables	
 *--------------------------------------------------------------------------*/

extern int        install_cmap ;
extern int        verbose ;
extern char      *module ;
extern boolean    no_interface ;


gint       wait_me ;


/*----------------------------------------------------------------------------
 *							Functions code
 *--------------------------------------------------------------------------*/

void
ccl_gtk_init (int ac, char **av)
{


   /* Initialize Gtk application */
   gtk_init (&ac, &av) ;

   cpk_init ("cxc_logo.ppm") ;

   /* Decode list arguments */
   app_init_update_status ("Initializing Display struct", NULL, 0.00);
   /* cxc_display_init (&display) ; */

   app_init_update_status ("Looking for data files", NULL, 0.10);
   /* cxc_arg_list_decode (ac, av) ; */
   /* app_init_update_status (NULL, "Reading file ...", 0.05); *****/

   app_init_update_status ("Initializing Frames", " ", 0.80);
   /* cxc_frames_init (&display) ; */

   app_init_update_status ("Building Main Window", NULL, 1.00);
   /* cxc_display_new (&display) ;
   cxc_main_window (&display) ; */

   wait_me = 0 ;

  /*  Things to do only if there is an interface  */
  if (no_interface == FALSE)
    {

    }

   /* Control */
   gtk_main () ;
}


