/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 					      Bologna Astronomical Observatory
 *----------------------------------------------------------------------------
 * File name 	: cataCal.c
 * Author 	: Paolo Montegriffo
 * Created on	: May, 2003	
 * Language	: ANSI C
 * Description	: 
 * Version      : 2.1.19 - May 10, 2003
 * Notes        : 
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							    Includes
 *--------------------------------------------------------------------------*/

#if HAVE_CONFIG_H
#  include "config.h"
#endif

#include "common.h"

#include <gtk/gtk.h>

#include "catalib.h"
#include "cp_messages.h"
#include "cpk_init.h"
#include "cpk_frame.h"
#include "cpk_buttons.h"
#include "cpk_gtk_util.h"

#include "ccl_main.h"
#include "ccl_decode.h"
#include "ccl_buttons.h"

#ifdef pippozzo
#include "catacal.h"
#endif

/*----------------------------------------------------------------------------
 * 							Function prototypes
 *--------------------------------------------------------------------------*/

static int         ccl_gtk_init                   (int               ac,
                                                   char            **av) ;
static void        ccl_display_init               (ccl_display      *display) ;
static void        ccl_main_window                (ccl_display      *display) ;

static gint        delete_event                   (GtkWidget        *widget,
                                                   GdkEvent         *event,
                                                   ccl_display      *display) ;
static void        destroy                        (GtkWidget        *widget,
                                                   gpointer          data) ;


/*----------------------------------------------------------------------------
 * 								Defines
 *--------------------------------------------------------------------------*/

#ifndef MIN
#define MIN(a,b) (((a) <= (b)) ? (a) : (b))
#endif
#ifndef MAX
#define MAX(a,b) (((a) >= (b)) ? (a) : (b))
#endif
#ifndef ABS
#define ABS(a) ((a) < 0 ? (-(a)) : (a))
#endif
#ifndef SQR
#define SQR(a) ((a)*(a))
#endif
#ifndef M_PI
#define M_PI 3.141592653589793238
#endif


/*----------------------------------------------------------------------------
 * 							Global variables	
 *--------------------------------------------------------------------------*/

char      command[2048] = "\0" ;
char     *module        = "CataCalc" ;
char     *version       = VERSION " - May, 2003" ;

int       verbose ;
boolean   no_interface = FALSE ;  /* Activate display for interactive matching */
boolean   install_cmap = FALSE ;
gint      wait_me ;


gdouble    tunits ;

/*----------------------------------------------------------------------------
 * 							Private functions
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 *							Functions code
 *--------------------------------------------------------------------------*/

int 
main (int ac, char **av)
{
   int        i, status = 1 ;

   strcpy (command, *av) ;
   for (i = 1; i < ac; i++) 
      sprintf (command, "%s %s", command, *(av+i)) ;

   ccl_arg_check_help (ac, av) ;

   /* Initialize Gtk application */
   status = ccl_gtk_init (ac, av) ;

   if (status == 0) 
      cp_message (1, 1, "Done...\n") ;
   else 
      cp_message (1, 1, "Execution failed...\n") ;

   exit (status) ;
}

static int
ccl_gtk_init (int ac, char **av)
{
   static ccl_display display ;

   /* Initialize Gtk application */
   gtk_init (&ac, &av) ;

   cpk_init ("cxc_logo.ppm") ;

   /* Decode list arguments */
   app_init_update_status ("Initializing Display struct", NULL, 0.00);
   ccl_display_init (&display) ;

   app_init_update_status ("Looking for data files", NULL, 0.10);
   /* cxc_arg_list_decode (ac, av) ; */
   /* app_init_update_status (NULL, "Reading file ...", 0.05); *****/

   app_init_update_status ("Initializing Frames", " ", 0.80);
   /* cxc_frames_init (&display) ; */

   app_init_update_status ("Building Main Window", NULL, 1.00);
   /* cxc_display_new (&display) ; */
   ccl_main_window (&display) ;

   wait_me = 0 ;

  /*  Things to do only if there is an interface  */
  if (no_interface == FALSE)
    {

    }

   /* Control */
   gtk_main () ;
   return 0 ;
}

static void
ccl_display_init (ccl_display *display)
{
   display->we_are_exiting = FALSE ;
}

static void
ccl_main_window (ccl_display *display)
{
   GtkWidget     *window, *vbox, *hbox, *box, *menu_bar, *vpaned ;
   CpkFrameSet   *fs ;
   CpkButtons    *bt ;
   GtkTooltips   *ttip ;

   set_font ("-*-times-medium-r-normal--17-140-*-*-*-*-*-*") ;
   ttip = gtk_tooltips_new () ;

   /* Create a new window */

   CPKDEBUG5 ("Creating the main window", 5) ;
   gtk_widget_push_visual (gdk_rgb_get_visual ()) ;
   gtk_widget_push_colormap (gdk_rgb_get_cmap ()) ;
   display->window = window = gtk_window_new (GTK_WINDOW_TOPLEVEL) ;
   gtk_widget_pop_visual () ;
   gtk_widget_pop_colormap () ;   
   gtk_window_set_title (GTK_WINDOW (window), module) ;
   gtk_container_set_border_width (GTK_CONTAINER (window), 0) ;
   gtk_widget_set_usize (window, 750, 800) ;

   gtk_window_set_policy (GTK_WINDOW (window), TRUE, TRUE, TRUE) ;
   gtk_signal_connect (GTK_OBJECT (window), "delete_event", 
                       GTK_SIGNAL_FUNC (delete_event), (gpointer)display) ;
   gtk_signal_connect (GTK_OBJECT (window), "destroy", 
                       GTK_SIGNAL_FUNC (destroy), NULL) ;

   /* Main container */
   vbox = gtk_vbox_new (FALSE, 0) ;
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 1) ;
   gtk_container_add (GTK_CONTAINER (window), vbox) ;
   gtk_widget_show (vbox) ;

   /* Create the menubar */
   CPKDEBUG5 ("Creating the menubar", 5) ;
   menu_bar = ccl_menu_main (window, display) ;
   gtk_box_pack_start (GTK_BOX (vbox), menu_bar, FALSE, FALSE, 0) ;
   gtk_widget_show (menu_bar) ;

   /* Top area */
   CPKDEBUG5 ("Creating Top area", 5) ;
   hbox = gtk_hbox_new (FALSE, 2) ;
   gtk_container_set_border_width (GTK_CONTAINER (hbox), 2) ;
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0) ;
   gtk_widget_show (hbox) ;

   display->fs = cpk_frame_set_new () ;
   fs = CPK_FRAME_SET (display->fs) ;
   fs->tile_mode = TILE_GRID ;
   fs->mode = FRAME_TILE ;

   /* Magnifier Panel */
   gtk_box_pack_end (GTK_BOX (hbox), fs->magnifier_box, FALSE, FALSE, 0) ;

   /* Paner Panel */
   gtk_box_pack_end (GTK_BOX (hbox), fs->paner_box, FALSE, FALSE, 0) ;

   /* Buttons area */
   CPKDEBUG5 ("Creating Buttons area", 5) ;
   display->bt = ccl_buttons_new (display) ;
   bt = CPK_BUTTONS (display->bt) ;
   gtk_container_set_border_width (GTK_CONTAINER (bt->box), 0) ;
   gtk_box_pack_start (GTK_BOX (vbox), bt->box, FALSE, FALSE, 0) ;
   gtk_widget_show_all (bt->box) ;

   /* Central area */
   vpaned = gtk_vpaned_new () ;
   gtk_box_pack_start (GTK_BOX (vbox), vpaned, TRUE, TRUE, 0) ;
   gtk_paned_set_handle_size (GTK_PANED (vpaned), 9) ;

   /* Frames */
   CPKDEBUG5 ("Creating Frames", 5) ;
   box = gtk_vbox_new (FALSE, 0) ;
   gtk_widget_show (box) ;
   gtk_paned_pack1 (GTK_PANED (vpaned), box, TRUE, TRUE) ;
   gtk_box_pack_start (GTK_BOX (box), display->fs, TRUE, TRUE, 0) ;
   gtk_widget_show (display->fs) ;

   /* Toolbar */

   /* Console area */
   display->console = hbox = cpk_console_new (window) ;
   gtk_paned_pack2 (GTK_PANED (vpaned), hbox, TRUE, TRUE) ; 
   gtk_paned_set_position (GTK_PANED (vpaned), 500) ;

   gtk_widget_show (vpaned) ;

   /* Show window */
   gtk_widget_show (window) ;

   /* Set frame mode througt main menu check item */

   return ;
}

static gint
delete_event (GtkWidget *widget, GdkEvent *event, ccl_display *display)
{
   /* if (CXC_DATA (display->data)->locked) {
      suspend = TRUE ;
      display->we_are_exiting = TRUE ;
      return TRUE ;
   } */
   return FALSE ;
}

static void
destroy (GtkWidget *widget, gpointer data)
{
   if (gtk_main_level ())
      gtk_main_quit () ;
   else
      gtk_exit (0) ;
}

void
ccl_quit_program (GtkWidget *widget, ccl_display *display)
{
   if (!display->we_are_exiting)
      if (cpk_confirm (display->window, "CataCalc: Exit", " Exit Program ? "))
         return ;

   /* Distrugge tutti i children */
   /* ... */

   gtk_widget_destroy (display->window) ;
}

