/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 					      Bologna Astronomical Observatory
 *----------------------------------------------------------------------------
 * File name 	: CataXcorr.c
 * Author 	: Paolo Montegriffo
 * Created on	: October, 2000	
 * Language	: ANSI C
 * Description	: 
 * Version      : 2.0 - November 21, 2000
 *--------------------------------------------------------------------------*/

#if HAVE_CONFIG_H
#  include "config.h"
#endif

#include "common.h"
#include "cp_messages.h"

#if HAVE_STDARG_H
#  include <stdarg.h>
#  define VA_START(a, f)        va_start(a, f)
#else
#  if HAVE_VARARGS_H
#    include <varargs.h>
#    define VA_START(a, f)      va_start(a)
#  endif
#endif
#ifndef VA_START
#   /* error no variadic api */
#endif

/*----------------------------------------------------------------------------
 * 							Global variables	
 *--------------------------------------------------------------------------*/

extern char *module;        /* Calling module */
extern int  verbose;        /* Verbosity: 0=silent */

/*----------------------------------------------------------------------------
 * 							Private functions
 *--------------------------------------------------------------------------*/

static void chop_r (char * s)
{ if (s) while (s[strlen (s)-1] == (char)'\n') s[strlen (s)-1]=(char)0 ; }

/*----------------------------------------------------------------------------
 *							Function codes
 *--------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
 * Function  : vm_message
 * In        : Flag to print or not module name, message to be printed out
 * Out       : Message on stdout
 * Purpouse  : Prints a message on stdout if verbose is 1 or more
 *---------------------------------------------------------------------------*/

/* void cp_message(int code, int v_level, char *fmt, ...) */
void
#if HAVE_STDARG_H
cp_message (int code, int v_level, const char *fmt, ...)
#else
cp_message (code, v_level, fmt, va_alist)
     int code ;
     int v_level ;
     const char *fmt ;
     va_dcl
#endif
{
   char   *msg ;
   char    ffmt[10];
   va_list ap ;

   (void) sprintf (ffmt, "%%%ds", (int) strlen (module)+2);
   if (!verbose)
      return;
   if (v_level>verbose)
      return;
   if (code == 1) (void) fprintf (stdout, "%s: ", module) ;
   else (void) fprintf (stdout, ffmt, " ") ;
   msg = (char*) strdup (fmt) ;

   VA_START (ap, fmt) ;
   (void) vfprintf (stdout, msg, ap) ;
   va_end (ap) ;
   free (msg) ;
   /* (void)fprintf(stdout, "\n") ; */
   (void) fflush (stdout) ;
}

/*-----------------------------------------------------------------------------
 * Function  : error
 * In        : Message to be printed out
 * Out       : Message on stderr
 * Purpouse  : Prints an error message on stderr
 *---------------------------------------------------------------------------*/

/* void cp_error(char *fmt, ...) */
void
#if HAVE_STDARG_H
cp_error (const char *fmt, ...)
#else
cp_error (fmt, va_alist)
     const char *fmt ;
     va_dcl
#endif
{
   char   *msg ;
   va_list ap ;
   (void) fprintf (stderr, "\7\7") ;
   (void) fprintf (stderr, "\n%s: ", module) ;
   (void) fprintf (stderr, "*** SEVERE: ") ;
   msg = (char*) strdup (fmt) ;
   chop_r (msg) ;
   VA_START (ap, fmt) ;
   (void) vfprintf (stderr, msg, ap) ;
   va_end (ap) ;
   free (msg) ;
   (void) fprintf (stderr, "\n") ;
   (void) fflush (stderr) ;
}

/*-----------------------------------------------------------------------------
 * Function  : warning
 * In        : Message to be printed out
 * Out       : Message on stderr
 * Purpouse  : Prints a warning message on stderr
 *---------------------------------------------------------------------------*/

/* void cp_warning(char *fmt, ...) */
void
#if HAVE_STDARG_H
cp_warning (const char *fmt, ...)
#else
cp_warning (fmt, va_alist)
     const char *fmt ;
     va_dcl
#endif
{
   char   *msg ;
   va_list ap ;
   (void) fprintf (stderr, "\7") ;
   (void) fprintf (stderr, "%s: ", module) ;
   (void) fprintf (stderr, "*** WARNING: ") ;
   msg = (char*) strdup (fmt) ;
   chop_r (msg) ;
   VA_START (ap, fmt) ;
   (void) vfprintf (stderr, msg, ap) ;
   va_end (ap) ;
   free (msg) ;
   (void) fprintf (stderr, "\n") ;
   (void) fflush (stderr) ;
}

