/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Includes
 *--------------------------------------------------------------------------*/

#if HAVE_CONFIG_H
#  include "config.h"
#endif

#include "common.h"

#include <gtk/gtk.h>

#include "cpk_gtk_util.h"
#include "cpk_plot.h"
#include "cpk_draw.h"


/*----------------------------------------------------------------------------
 * 							Global variables	
 *--------------------------------------------------------------------------*/

/* Local data */

static GtkWidgetClass  *parent_class = NULL ;

/*----------------------------------------------------------------------------
 * 							Function prototypes
 *--------------------------------------------------------------------------*/
                                               
static void        cpk_draw_class_init       (CpkDrawClass      *klass) ;

static void        cpk_draw_init             (CpkDraw           *draw) ;
 

/*----------------------------------------------------------------------------
 *							Function codes
 *--------------------------------------------------------------------------*/

GtkType
cpk_draw_get_type (void)
{
  static GtkType draw_type = 0;

  if (!draw_type)
    {
      static const GtkTypeInfo draw_info =
      {
	"CpkDraw",
	sizeof (CpkDraw),
	sizeof (CpkDrawClass),
	(GtkClassInitFunc) cpk_draw_class_init,
	(GtkObjectInitFunc) cpk_draw_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
        (GtkClassInitFunc) NULL,
      };

      draw_type = gtk_type_unique (GTK_TYPE_WIDGET, &draw_info);
    }

  return draw_type ;
}

static void
cpk_draw_class_init (CpkDrawClass *class)
{
   GtkObjectClass    *object_class ;
   GtkWidgetClass    *widget_class ;

   object_class = (GtkObjectClass*) class ;
   widget_class = (GtkWidgetClass*) class ;

   parent_class = gtk_type_class (GTK_TYPE_WIDGET) ;

   class->x = NULL ;
   class->y = NULL ;
   class->draw_geometry = NULL ;
   class->draw_button_press = NULL ;
   class->draw_paint = NULL ;
   class->draw_pan = NULL ;
   class->draw_pan_border = cpk_draw_pan_border ;
   class->draw_mgf = NULL ;
   class->world2draw = NULL ;
   class->draw2world = NULL ;
   class->world2pan = NULL ;
   class->pan2world = NULL ;
   class->world2mgf = NULL ;
}

static void
cpk_draw_init (CpkDraw *draw)
{
   draw->nx = 0 ;
   draw->ny = 0 ;
   GTK_WIDGET_SET_FLAGS (draw, GTK_NO_WINDOW) ;
}

void
cpk_draw_world_to_draw (CpkDraw    *draw,
                        gdouble     xw,
                        gdouble     yw,
                        gdouble    *xd,
                        gdouble    *yd)
{
   CpkDrawClass *klass ;

   g_return_if_fail (draw != NULL) ;
   g_return_if_fail (CPK_IS_DRAW (draw)) ;

   klass = CPK_DRAW_CLASS (GTK_OBJECT (draw)->klass) ;

   klass->world2draw (draw, xw, yw, xd, yd) ;
}

void
cpk_draw_draw_to_world (CpkDraw    *draw,
                        gdouble     xd,
                        gdouble     yd,
                        gdouble    *xw,
                        gdouble    *yw)
{
   CpkDrawClass *klass ;

   g_return_if_fail (draw != NULL) ;
   g_return_if_fail (CPK_IS_DRAW (draw)) ;

   klass = CPK_DRAW_CLASS (GTK_OBJECT (draw)->klass) ;

   klass->draw2world (draw, xd, yd, xw, yw) ;
}

void
cpk_draw_world_to_pan (CpkDraw    *draw,
                       gdouble     xw,
                       gdouble     yw,
                       gdouble    *xp,
                       gdouble    *yp)
{
   CpkDrawClass *klass ;

   g_return_if_fail (draw != NULL) ;
   g_return_if_fail (CPK_IS_DRAW (draw)) ;

   klass = CPK_DRAW_CLASS (GTK_OBJECT (draw)->klass) ;

   klass->world2pan (draw, xw, yw, xp, yp);
}

void
cpk_draw_pan_to_world (CpkDraw    *draw,
                       gdouble     xp,
                       gdouble     yp,
                       gdouble    *xw,
                       gdouble    *yw)
{
   CpkDrawClass *klass ;

   g_return_if_fail (draw != NULL) ;
   g_return_if_fail (CPK_IS_DRAW (draw)) ;

   klass = CPK_DRAW_CLASS (GTK_OBJECT (draw)->klass) ;

   klass->pan2world (draw, xp, yp, xw, yw);
}

void
cpk_draw_world_to_mgf (CpkDraw    *draw,
                       gdouble     xw,
                       gdouble     yw,
                       gdouble    *xm,
                       gdouble    *ym)
{
   CpkDrawClass *klass ;

   g_return_if_fail (draw != NULL) ;
   g_return_if_fail (CPK_IS_DRAW (draw)) ;

   klass = CPK_DRAW_CLASS (GTK_OBJECT (draw)->klass) ;

   klass->world2mgf (draw, xw, yw, xm, ym);
}

void
cpk_draw_geometry (CpkDraw *draw, GtkWidget *darea, gdouble x, gdouble y)
{
   CpkDrawClass *klass ;
   g_return_if_fail (draw != NULL) ;
   g_return_if_fail (CPK_IS_DRAW (draw)) ;
   g_return_if_fail (GTK_WIDGET_VISIBLE (darea)) ;
   klass = CPK_DRAW_CLASS (GTK_OBJECT (draw)->klass) ;

   klass->draw_geometry (draw, darea, x, y) ;
}

void
cpk_draw_pan_border (CpkDraw *draw, GtkWidget *darea)
{
   CpkPlot *plot ;
   gdouble  x, y, x1, y1, x2, y2 ;
   static GdkGC *border_gc = NULL ;

   g_return_if_fail (draw != NULL) ;
   g_return_if_fail (CPK_IS_DRAW (draw)) ;
   g_return_if_fail (GTK_WIDGET_VISIBLE (darea)) ;
   
   plot = CPK_PLOT (GTK_WIDGET (draw)->parent) ;

   if (!border_gc) {
      border_gc = gdk_gc_new (plot->parent->window) ;
      cpk_set_fg_color (plot->parent, border_gc, "blue") ;
   }

   /* Plot frame border */

   cpk_draw_draw_to_world (draw, 0.0, 0.0, &x, &y) ;
   cpk_draw_world_to_pan (draw, x, y, &x1, &y1) ;
   cpk_draw_draw_to_world (draw, (gdouble)draw->nx-1, (gdouble)draw->ny-1, &x, &y) ;
   cpk_draw_world_to_pan (draw, x, y, &x2, &y2) ;

   if (CPK_PLOT_PAN_ACTIVE (plot)) 
      gdk_gc_set_line_attributes (border_gc, 2,
            GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_MITER) ;
   else
      gdk_gc_set_line_attributes (border_gc, 1,
            GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_MITER) ;

   gdk_draw_rectangle (darea->window, border_gc, FALSE, 
         (gint)x1+1, (gint)y1+1, (gint)(x2-x1), (gint)(y2-y1)) ;
}

void
cpk_draw_button_press (CpkDraw   *draw, 
                       GtkWidget *darea, 
                       gdouble    x, 
                       gdouble    y, 
                       GdkModifierType state)
{
   CpkDrawClass *klass ;
   g_return_if_fail (draw != NULL) ;
   g_return_if_fail (CPK_IS_DRAW (draw)) ;
   g_return_if_fail (GTK_WIDGET_VISIBLE (darea)) ;
   klass = CPK_DRAW_CLASS (GTK_OBJECT (draw)->klass) ;

   klass->draw_button_press (draw, darea, x, y, state) ;
}

void
cpk_draw_button_release (CpkDraw *draw, GtkWidget *darea, gdouble x, gdouble y)
{
   /* esempio buono per cxc_map */
   /* qui cxcmap richiama geometry() : serve ? */

}

void
cpk_draw_paint (CpkDraw *draw, GdkPixmap *pixmap, gint moving) 
{
   CpkDrawClass *klass ;
   g_return_if_fail (draw != NULL) ;
   g_return_if_fail (CPK_IS_DRAW (draw)) ;
   klass = CPK_DRAW_CLASS (GTK_OBJECT (draw)->klass) ;

   klass->draw_paint (draw, pixmap, moving) ;
}


gint
cpk_draw_find_nearest (CpkDraw *draw, gdouble x, gdouble y)
{
   CpkDrawClass *klass ;
   gint    i, id = -1 ;
   gdouble sqdist, near, xdist, ydist ;
   near = 30*30 ;

   klass = CPK_DRAW_CLASS (GTK_OBJECT (draw)->klass) ;

   CPKDEBUG5 ("cpk_draw_find_nearest", 6) ;

   for (i = 0; i < draw->n; i++) {
      xdist = klass->x (draw, i) - x ;
      ydist = klass->y (draw, i) - y ;
      sqdist = xdist*xdist + ydist*ydist ;
      if (sqdist <= near) {
         near = sqdist;
         id = i ;
      }
   }
   return id ;
}

void
cpk_draw_draw_mark (CpkDraw *draw, GtkWidget *widget, gint id, gboolean emit)
{
   CpkDrawClass *klass ;
   GdkRectangle  update_rect ;
   GdkPoint      point[10] ;
   gdouble       xd, yd ;
   static GdkGC *border_gc = NULL ;

   g_return_if_fail (draw != NULL) ;
   g_return_if_fail (CPK_IS_DRAW (draw)) ;
   g_return_if_fail (widget != NULL) ;
   g_return_if_fail (GTK_IS_DRAWING_AREA (widget)) ;

   klass = CPK_DRAW_CLASS (GTK_OBJECT (draw)->klass) ;
   if (id == draw->marked)
      return ;

   /* Clear old mark */
   if (draw->marked > 0) {
      cpk_draw_world_to_draw (draw, 
            klass->x (draw, draw->marked), klass->y (draw, draw->marked), &xd, &yd) ;
      update_rect.x = (gint)xd - 15 ;
      update_rect.y = (gint)yd - 15 ;
      update_rect.width = update_rect.height = 31 ;
      gtk_widget_draw (widget, &update_rect) ;
   }
   draw->marked = id ;

   /* Draw new mark */
   if (draw->marked > 0) {
      if (!border_gc) {
         border_gc = gdk_gc_new (widget->window) ;
         cpk_set_fg_color (widget, border_gc, "blue") ;
         gdk_gc_set_line_attributes (border_gc, 1, GDK_LINE_SOLID, 
               GDK_CAP_PROJECTING, GDK_JOIN_ROUND) ;
      }
      cpk_draw_world_to_draw (draw, 
            klass->x (draw, draw->marked), klass->y (draw, draw->marked), &xd, &yd) ;
      cpk_poly_plus (point, (gint)xd, (gint)yd, 0, 15) ;
      gdk_draw_polygon (widget->window, border_gc, FALSE, point, 9) ;
   }
   /* if (emit)
      gtk_signal_emit_by_name (GTK_OBJECT (draw), "mark-changed") ; */

}





