/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

/* Derived from GIMP - app/cursoutil.h */

/* The GIMP -- an image manipulation program
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "config.h"

#include <gtk/gtk.h>

#include "cpk_gtk_cursorutil.h"

#include "pixmaps/hand.xbm"
#include "pixmaps/hand_mask.xbm"
#include "pixmaps/measure_small.xbm"
#include "pixmaps/measure_small_mask.xbm"
#include "pixmaps/zoom.xbm"
#include "pixmaps/zoom_mask.xbm"
#include "pixmaps/rotate_small.xbm"
#include "pixmaps/rotate_small_mask.xbm"
#include "pixmaps/scissors_small.xbm"
#include "pixmaps/scissors_small_mask.xbm"
#include "pixmaps/plus.xbm"
#include "pixmaps/plus_mask.xbm"
#include "pixmaps/minus.xbm"
#include "pixmaps/minus_mask.xbm"


static BitmapCursor cpk_cursors[] =
/* these have to match up with the enum in cursorutil.h */
{
  {
    hand_bits, hand_mask_bits,
    hand_width, hand_height,
    hand_x_hot, hand_y_hot, NULL, NULL, NULL
  },
  {
    zoom_bits, zoom_mask_bits,
    zoom_width, zoom_height,
    zoom_x_hot, zoom_y_hot, NULL, NULL, NULL
  },
  {
    rotate_small_bits, rotate_small_mask_bits,
    rotate_small_width, rotate_small_height,
    rotate_small_x_hot, rotate_small_y_hot, NULL, NULL, NULL
  },
  {
    measure_small_bits, measure_small_mask_bits,
    measure_small_width, measure_small_height,
    measure_small_x_hot, measure_small_y_hot, NULL, NULL, NULL
  },
  {
    scissors_small_bits, scissors_small_mask_bits,
    scissors_small_width, scissors_small_height,
    scissors_small_x_hot, scissors_small_y_hot, NULL, NULL, NULL
  }
};

enum
{
  CPK_PLUS_CURSOR = CPK_LAST_CURSOR_ENTRY + 1,
  CPK_MINUS_CURSOR
} ;

static BitmapCursor modifier_cursors[] =
/* these have to match up with the enum above */
{
  {
    plus_bits, plus_mask_bits,
    plus_width, plus_height,
    plus_x_hot, plus_y_hot, NULL, NULL, NULL
  },
  {
    minus_bits, minus_mask_bits,
    minus_width, minus_height,
    minus_x_hot, minus_y_hot, NULL, NULL, NULL
  }
};


extern GSList   *display_list; /* It's in gdisplay.c, FYI */
static gboolean  pending_removebusy = FALSE;


static void
create_cursor_bitmaps (BitmapCursor *bmcursor)
{
  if (bmcursor->bitmap == NULL)
    bmcursor->bitmap = gdk_bitmap_create_from_data (NULL, (gchar*)bmcursor->bits,
						    bmcursor->width,
						    bmcursor->height);
  g_return_if_fail (bmcursor->bitmap != NULL);

  if (bmcursor->mask == NULL)
    bmcursor->mask = gdk_bitmap_create_from_data (NULL, (gchar*)bmcursor->mask_bits,
						  bmcursor->width,
						  bmcursor->height);
  g_return_if_fail (bmcursor->mask != NULL);
}

static void
create_cursor (BitmapCursor *bmcursor)
{
  if (bmcursor->bitmap == NULL ||
      bmcursor->mask == NULL)
    create_cursor_bitmaps (bmcursor);

  if (bmcursor->cursor == NULL)
    {
      GdkColor fg, bg;

      /* should have a way to configure the mouse colors */
      gdk_color_parse ("#FFFFFF", &bg);
      gdk_color_parse ("#000000", &fg);

      bmcursor->cursor = gdk_cursor_new_from_pixmap (bmcursor->bitmap,
						     bmcursor->mask,
						     &fg, &bg,
						     bmcursor->x_hot,
						     bmcursor->y_hot);
    }

  g_return_if_fail (bmcursor->cursor != NULL);
}

static void
cpk_change_win_cursor (GdkWindow      *win,
			CpkCursorType   curtype,
			CursorModifier  modifier)
{
  GdkCursor      *cursor;
  CpkCursorType  modtype = CPK_PLUS_CURSOR;

  GdkBitmap *bitmap;
  GdkBitmap *mask;

  GdkColor   color;
  GdkColor   fg, bg;

  static GdkGC *gc = NULL;

  gint width;
  gint height;

  BitmapCursor *bmcursor   = NULL;
  BitmapCursor *bmmodifier = NULL;
  BitmapCursor *bmtool     = NULL;

  g_return_if_fail (curtype < CPK_LAST_CURSOR_ENTRY);


  curtype -= CPK_PAN_CURSOR;
  bmcursor = &cpk_cursors[(int)curtype];

 switch (modifier)
    {
    case CURSOR_MODIFIER_PLUS:
      modtype = CPK_PLUS_CURSOR;
      break;
    case CURSOR_MODIFIER_MINUS:
      modtype = CPK_MINUS_CURSOR;
      break;
    default:
      break;
    }

  if (modifier != CURSOR_MODIFIER_NONE)
    {
      modtype -= CPK_PLUS_CURSOR;
      bmmodifier = &modifier_cursors[(int)modtype];
    }

  if (bmcursor->bitmap == NULL ||
      bmcursor->mask == NULL)
    create_cursor_bitmaps (bmcursor);

  if (bmmodifier &&
      (bmmodifier->bitmap == NULL ||
       bmmodifier->mask == NULL))
    create_cursor_bitmaps (bmmodifier);

  if (gc == NULL)
    gc = gdk_gc_new (bmcursor->bitmap);

  gdk_window_get_size (bmcursor->bitmap, &width, &height);

  bitmap = gdk_pixmap_new (NULL, width, height, 1);
  mask   = gdk_pixmap_new (NULL, width, height, 1);

  color.pixel = 1;
  gdk_gc_set_foreground (gc, &color);

  gdk_draw_pixmap (bitmap, gc, bmcursor->bitmap,
                   0, 0, 0, 0, width, height);
  if (bmmodifier)
    {
      gdk_gc_set_clip_mask (gc, bmmodifier->bitmap);
      gdk_draw_pixmap (bitmap, gc, bmmodifier->bitmap,
                       0, 0, 0, 0, width, height);
      gdk_gc_set_clip_mask (gc, NULL);
    }

  gdk_draw_pixmap (mask, gc, bmcursor->mask,
                   0, 0, 0, 0, width, height);
  if (bmmodifier)
    {
      gdk_gc_set_clip_mask (gc, bmmodifier->mask);
      gdk_draw_pixmap (mask, gc, bmmodifier->mask,
                       0, 0, 0, 0, width, height);
      gdk_gc_set_clip_mask (gc, NULL);
    }

  /* should have a way to configure the mouse colors */
  gdk_color_parse ("#FFFFFF", &bg);
  gdk_color_parse ("#000000", &fg);

  cursor = gdk_cursor_new_from_pixmap (bitmap, mask,
				       &fg, &bg,
				       bmcursor->x_hot,
				       bmcursor->y_hot);
  gdk_window_set_cursor (win, cursor);
  gdk_cursor_destroy (cursor);
  gdk_bitmap_unref (bitmap);
  gdk_bitmap_unref (mask);
}

void
change_win_cursor (GdkWindow      *win,
		   GdkCursorType   cursortype,
		   CursorModifier  modifier)
{
  GdkCursor *cursor;
  
  if (cursortype > GDK_LAST_CURSOR)
    {
      cpk_change_win_cursor (win, (CpkCursorType) cursortype,
			      modifier);
      return;
    }

  cursor = gdk_cursor_new (cursortype);
  gdk_window_set_cursor (win, cursor);
  gdk_cursor_destroy (cursor);
}

void
unset_win_cursor (GdkWindow *win)
{
  gdk_window_set_cursor (win, NULL);
}

#ifdef cippa

void
gimp_add_busy_cursors_until_idle (void)
{
  if (!pending_removebusy)
    {
      gimp_add_busy_cursors ();
      gtk_idle_add_priority (GTK_PRIORITY_HIGH,
			     gimp_remove_busy_cursors, NULL);
      pending_removebusy = TRUE;
    }
}
     
void
gimp_add_busy_cursors (void)
{
  GDisplay *gdisp;
  GSList   *list;

  /* FIXME: gimp_busy HACK */
  gimp_busy = TRUE;

  /* Canvases */
  for (list = display_list; list; list = g_slist_next (list))
    {
      gdisp = (GDisplay *) list->data;
      gdisplay_install_override_cursor (gdisp, GDK_WATCH);
    }

  /* Dialogs */
  dialog_idle_all ();

  gdk_flush ();
}

gint
gimp_remove_busy_cursors (gpointer data)
{
  GDisplay *gdisp;
  GSList   *list;

  /* Canvases */
  for (list = display_list; list; list = g_slist_next (list))
    {
      gdisp = (GDisplay *) list->data;
      gdisplay_remove_override_cursor (gdisp);
    }

  /* Dialogs */
  dialog_unidle_all ();

  pending_removebusy = FALSE;

  /* FIXME: gimp_busy HACK */
  gimp_busy = FALSE;

  return 0;
}
#endif

/***************************************************************/
/* gtkutil_compress_motion:

   This function walks the whole GDK event queue seeking motion events
   corresponding to the widget 'widget'.  If it finds any it will
   remove them from the queue, write the most recent motion offset
   to 'lastmotion_x' and 'lastmotion_y', then return TRUE.  Otherwise
   it will return FALSE and 'lastmotion_x' / 'lastmotion_y' will be
   untouched.
 */
/* The gtkutil_compress_motion function source may be re-used under
   the XFree86-style license. <adam@gimp.org> */
gboolean
gtkutil_compress_motion (GtkWidget *widget,
			 gdouble   *lastmotion_x,
			 gdouble   *lastmotion_y)
{
  GdkEvent *event;
  GList    *requeued_events = NULL;
  GList    *list;
  gboolean  success = FALSE;

  /* Move the entire GDK event queue to a private list, filtering
     out any motion events for the desired widget. */
  while (gdk_events_pending ())
    {
      event = gdk_event_get ();

      if (!event)
	{
	  /* Do nothing */
	}
      else if ((gtk_get_event_widget (event) == widget) &&
	       (event->any.type == GDK_MOTION_NOTIFY))
	{
	  *lastmotion_x = event->motion.x;
	  *lastmotion_y = event->motion.y;
	  
	  gdk_event_free (event);
	  success = TRUE;
	}
      else
	{
	  requeued_events = g_list_prepend (requeued_events, event);
	}
    }
  
  /* Replay the remains of our private event list back into the
     event queue in order. */

  requeued_events = g_list_reverse (requeued_events);

  for (list = requeued_events; list; list = g_list_next (list))
    {
      gdk_event_put ((GdkEvent*) list->data);
      gdk_event_free ((GdkEvent*) list->data);
    }
  
  g_list_free (requeued_events);

  return success;
}
