/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

#ifndef CPK_PLOT_H
#define CPK_PLOT_H 1

#include "common.h"

BEGIN_C_DECLS

#include <gdk/gdk.h>
#include <gtk/gtkbin.h>
/* #include "cpk_gtk_util.h" */

#define TYPE_CPK_PLOT                (cpk_plot_get_type ())
#define CPK_PLOT(obj)                (GTK_CHECK_CAST ((obj), TYPE_CPK_PLOT, CpkPlot))
#define CPK_PLOT_CLASS(klass)        (GTK_CHECK_CLASS_CAST ((klass), TYPE_CPK_PLOT, CpkPlotClass))
#define CPK_IS_PLOT(obj)             (GTK_CHECK_TYPE ((obj), TYPE_CPK_PLOT))
#define CPK_IS_PLOT_CLASS(klass)     (GTK_CHECK_CLASS_TYPE ((klass), TYPE_CPK_PLOT))

/*** Rivedere dove si puo mettere il seguente enum...***/
enum {
   POINTER = 0,
   SELECT,
   PAN,
   ZOOM,
   ROTATE,
   RULER,
   CROSSHAIR, 
   NOACT
} ;

typedef enum
{
  CPK_CONFIGURE         = 1 << 0,   /* Riconfigure la pixmap */
  CPK_PAINT             = 1 << 1,   /* Riesegue Plot */
  CPK_SCALE             = 1 << 2,   /* Ricalcola parametri Plot */
  CPK_LABEL             = 1 << 3,
  CPK_CENTER            = 1 << 4,   /* Plot croce centrale */
  CPK_BORDER            = 1 << 5,
  CPK_CONFIGURE_PAN     = 1 << 6,
  CPK_CONFIGURE_MGF     = 1 << 7,
  CPK_PAINT_PAN         = 1 << 8,
  CPK_PAINT_MGF         = 1 << 9,
  CPK_DRAW              = 1 << 10,   /* Draw attivo */
  CPK_GRABBED           = 1 << 11,   /* Plot in fase di trascinamento */
  CPK_PAN_ACTIVE        = 1 << 12,
  CPK_NO_MOTION         = 1 << 13,   /* sospende motion_notify_event */
  CPK_MOVING            = 1 << 14,   /* Plot in movimento */
  CPK_BLINKING          = 1 << 15
} CpkPlotFlags;

#define CPK_PLOT_FLAGS(obj)          (CPK_PLOT (obj)->flag)
#define CPK_PLOT_CONFIGURE(obj)      ((CPK_PLOT_FLAGS (obj) & CPK_CONFIGURE) != 0)
#define CPK_PLOT_PAINT(obj)          ((CPK_PLOT_FLAGS (obj) & CPK_PAINT) != 0)
#define CPK_PLOT_SCALE(obj)          ((CPK_PLOT_FLAGS (obj) & CPK_SCALE) != 0)
#define CPK_PLOT_LABEL(obj)          ((CPK_PLOT_FLAGS (obj) & CPK_LABEL) != 0)
#define CPK_PLOT_CENTER(obj)         ((CPK_PLOT_FLAGS (obj) & CPK_CENTER) != 0)
#define CPK_PLOT_BORDER(obj)         ((CPK_PLOT_FLAGS (obj) & CPK_BORDER) != 0)
#define CPK_PLOT_CONFIGURE_PAN(obj)  ((CPK_PLOT_FLAGS (obj) & CPK_CONFIGURE_PAN) != 0)
#define CPK_PLOT_CONFIGURE_MGF(obj)  ((CPK_PLOT_FLAGS (obj) & CPK_CONFIGURE_MGF) != 0)
#define CPK_PLOT_PAINT_PAN(obj)      ((CPK_PLOT_FLAGS (obj) & CPK_PAINT_PAN) != 0)
#define CPK_PLOT_PAINT_MGF(obj)      ((CPK_PLOT_FLAGS (obj) & CPK_PAINT_MGF) != 0)
#define CPK_PLOT_DRAW(obj)           ((CPK_PLOT_FLAGS (obj) & CPK_DRAW) != 0)
#define CPK_PLOT_GRABBED(obj)        ((CPK_PLOT_FLAGS (obj) & CPK_GRABBED) != 0)
#define CPK_PLOT_PAN_ACTIVE(obj)     ((CPK_PLOT_FLAGS (obj) & CPK_PAN_ACTIVE) != 0)
#define CPK_PLOT_NO_MOTION(obj)      ((CPK_PLOT_FLAGS (obj) & CPK_NO_MOTION) != 0)
#define CPK_PLOT_MOVING(obj)         ((CPK_PLOT_FLAGS (obj) & CPK_MOVING) != 0)
#define CPK_PLOT_BLINKING(obj)       ((CPK_PLOT_FLAGS (obj) & CPK_BLINKING) != 0)

/* Macros for setting and clearing bits in the object_flags field of CpkPlot.
 */
#define CPK_PLOT_SET_FLAGS(obj,flag)    G_STMT_START{ (CPK_PLOT_FLAGS (obj) |= (flag)); }G_STMT_END
#define CPK_PLOT_UNSET_FLAGS(obj,flag)  G_STMT_START{ (CPK_PLOT_FLAGS (obj) &= ~(flag)); }G_STMT_END

#define CPK_PLOT_REMAP_LATER(obj)    (CPK_PLOT (obj)->nx = 0)
#define CPK_PLOT_SET_RESCALING(obj)  (CPK_PLOT (obj)->dx = 0.0)

enum {
   PLOT_LAST_SIGNAL
} ;

typedef struct _CpkPlot	CpkPlot ;
typedef struct _CpkPlotClass  CpkPlotClass ;

struct _CpkPlot
{
   GtkBin           bin ;
   gchar           *label;
   GtkWidget       *parent ;
   GdkPixmap       *pixmap ;
   GdkPixmap       *pixmap_pan ;
   GdkPixmap       *pixmap_mgf ;
   GdkGC           *gc[6] ;
   guint16          flag ;
   guint32          timer_1 ;
   guint32          timer_2 ;
   gint             crs_x, crs_y ;
   GdkModifierType  state;
   guint            p_action ;

} ;

struct _CpkPlotClass
{
   GtkBinClass parent_class ;
} ;

extern GtkType     cpk_plot_get_type          (void) ;
extern GtkWidget  *cpk_plot_new               (GtkWidget         *widget, 
                                               const gchar       *label) ;
extern gint        cpk_plot_pan_configure     (GtkWidget         *widget,
		                               GdkEventConfigure *event,
                                               CpkPlot           *plot) ;
extern gint        cpk_plot_pan_expose        (GtkWidget         *widget,
		                               GdkEventExpose    *event, 
                                               CpkPlot           *plot) ;
extern gint        cpk_plot_mgf_configure     (GtkWidget         *widget,
		                               GdkEventConfigure *event,
                                               CpkPlot           *plot) ;
extern gint        cpk_plot_mgf_expose        (GtkWidget         *widget,
		                               GdkEventExpose    *event, 
                                               CpkPlot           *plot) ;

END_C_DECLS

#endif /* !CPK_PLOT_H */
