/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/


#ifndef CXC_DATA_H
#define CXC_DATA_H 1

#include "common.h"

BEGIN_C_DECLS

#include <gdk/gdk.h>
#include <gtk/gtkdata.h>

#include "cataxcorr.h"
#include "platemodel.h"

#define TYPE_CXC_DATA                (cxc_data_get_type ())
#define CXC_DATA(obj)                (GTK_CHECK_CAST ((obj), TYPE_CXC_DATA, CxcData))
#define CXC_DATA_CLASS(klass)        (GTK_CHECK_CLASS_CAST ((klass), TYPE_CXC_DATA, CxcDataClass))
#define IS_CXC_DATA(obj)             (GTK_CHECK_TYPE ((obj), TYPE_CXC_DATA))
#define IS_CXC_DATA_CLASS(klass)     (GTK_CHECK_CLASS_TYPE ((klass), TYPE_CXC_DATA))

enum {
   FILE_STOP,
   FILE_NEXT,
   STAR_ALIGNED,
   BASE_FOUND,   /* aggiungere segnale di proiezione */
   MODEL_FIT,
   STACK_ADDED,
   DATA_CHANGED,
   DATA_LAST_SIGNAL
} ;

typedef struct _CxcData	CxcData ;
typedef struct _CxcDataClass  CxcDataClass ;

typedef struct _ProgressData {
    GtkWidget *box ; /* ***FIXME: useless */
    GtkWidget *icon ;
    GtkWidget *on ;
    GtkWidget *off ;
    guint      timer;
} ProgressData;

struct _CxcData 
{
   GtkData        data ;
   FileSet       *f_set ;
   g_pars        *gparam ;
   Finfo        **info ;
   StarList     **list ;
   PlateMod     **model ;
   gint           n ;

   StarList      *list1 ;
   StarList      *list2 ;
   PlateMod      *pmodel ;
   gint           nref ;
   gint           frm1, frm2 ; /***FIXME: ridondante ? */
   gboolean      *active ;
   StackTab      *stack ;

   gdouble        srad_pix ;  /* Mean map star radius for pixel units */
   gdouble        srad_deg ;  /* Mean map star radius for deg units */

   gboolean       locked ;
   GtkWidget     *stop ;
   GtkUpdateType  policy ;
   gint           timer_active ;
   ProgressData  *activity ;
   void          *p ;
} ;

struct _CxcDataClass
{
   GtkDataClass parent_class ;
   void (* file_stop)    (CxcData *data) ;
   void (* file_next)    (CxcData *data) ;
   void (* star_align)   (CxcData *data) ;
   void (* base_found)   (CxcData *data) ;
   void (* model_fit)    (CxcData *data) ;
   void (* stack_added)  (CxcData *data) ;
   void (* data_changed) (CxcData *data) ;
} ;

extern  GtkType	   cxc_data_get_type           (void) ;
extern  GtkData   *cxc_data_new	               (FileSet       *f_set,
                                                def_pars      *param,
                                                g_pars        *gparam) ;
extern  gint       cxc_data_remove_by_number   (CxcData       *data, 
                                                int            n) ;
extern  void       cxc_data_file_next          (GtkWidget     *widget,
                                                CxcData       *data) ;
extern  gint       cxc_data_file_select        (CxcData       *data, 
                                                guint          n) ;
extern  void       cxc_data_model_align        (GtkWidget     *widget,
                                                CxcData       *data) ;
extern  void       cxc_data_register           (GtkWidget     *widget,
                                                CxcData       *data) ;
extern  void       cxc_data_model_fit          (GtkWidget     *widget,
                                                CxcData       *data) ;
extern  gint       cxc_data_stack_init         (GtkWidget     *widget,
                                                CxcData       *data) ;
extern  void       cxc_data_stack_remove       (CxcData       *data, 
                                                gint           n) ;
extern  void       cxc_data_coinc_save         (GtkWidget     *widget,
                                                CxcData       *data) ;
extern  void       cxc_data_rot_save           (GtkWidget     *widget,
                                                CxcData       *data) ;
extern  void       cxc_data_model_save         (GtkWidget     *widget,
                                                CxcData       *data) ;
extern  void	   cxc_data_changed            (CxcData       *data) ;

extern  void	   cxc_data_reset              (GtkWidget     *widget,
                                                CxcData       *data) ;
extern  void       cxc_data_set_update_policy  (CxcData       *data,
                                                GtkUpdateType  policy) ;

extern  void       cxc_data_timeout_remove     (gpointer       data) ;


END_C_DECLS

#endif /* !CXC_DATA_H */
