/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

#ifndef CXC_GTK_MAIN_H
#define CXC_GTK_MAIN_H 1

#include "cxc_data.h"
#include "cxc_map.h"
#include "cxc_residuals.h"

#define  SHOW_COLOR  (*map->mode != FRAME_SINGLE && *map->mode != FRAME_MERGE) 

enum mode_tag {
   FRAME_SINGLE = 0,
   FRAME_PAIR,
   FRAME_TILE,
   FRAME_BLINK,
   FRAME_MERGE, 
   FRAME_COINC
} ;

typedef struct info_tag cxc_info ;

struct info_tag {
   GtkWidget *labels[15] ;
   char   file[31] ;
   char   id_1[31] ;
   char   id_2[33] ;
   char   xwl[3] ;
   char   ywl[4] ;
   char   xil[3] ;
   char   yil[4] ;
   char   xw[15] ;
   char   yw[15] ;
   char   xi[15] ;
   char   yi[15] ;
   char   frame[10] ;
   char   zoom[13] ;
   char   ang[13] ;
} ;

typedef struct cxc_disp_tag {

   GtkWidget  *window ;

   /* Data */
   GtkData    *data ;
   gboolean    we_are_exiting ;

   /* Star Maps */
   GtkWidget  **map ;

   /* Panels */
   GtkWidget  *flag_check[5] ;
   gboolean    settings[5] ;
   GtkWidget  *panels[5] ;

   /* Buttons */
   GtkWidget  *buttons[6] ;
   GtkWidget  *boxbuttons2 ;
   gint        nsel ;
   GtkWidget  *lastboxbuttons2 ;

   /* Info Panel */
   cxc_info   *info ;
   guint       wcs_fmt ;
   GtkWidget  *popup ;

   /* Paner */
   GtkWidget **paner ;

   /* Magnifier */
   GtkWidget  *magnifier ;
   guint       mgf_factor ;
   guint32     mgf_timer;

   /* Frames */
   GtkWidget  *frame_box ;
   GtkWidget  *frame_hbox ;
   GtkWidget  *frame_vbox ;
   GtkWidget  *frame_tbox ;
   guint       tbox_nrow, tbox_ncol ;
   GtkWidget **frames ;
   GtkWidget  *rframe ;
   gint        mode ;
   guint       nframes ;
   guint       user_reference ;
   guint       tile_mode ;
   guint       last_tile_mode ;
   gboolean    f_lock ;
   guint32     timer_lock ;
   guint      *show ;
   guint      *blink ;
   guint       focused ;        /* frame focalizzato */
   guint       current ;        /* frame attivo */
   guint       old_current ;
   PointerAct  pointer_action ;
   /* Timer for blink mode */
   guint32     timer;
   guint       blink_rate ;

   /* Toolbar */
   GtkWidget  *activity ; /***FIXME: put in data struct ?? */
   GtkWidget  *statusbar ;
   GtkWidget  *contrast ;
   GtkWidget  *luminosity ;
   GtkWidget  *savelog ;
   GtkWidget  *print ;
   GtkWidget  *stop ;

   /* Console */
   GtkWidget *console ;

   /* Dialogs */
   GtkWidget *data_content_dialog ;
   GtkWidget *auto_exec_dialog ;
   GtkWidget *align_frames_dialog ; /* find match */
   GtkWidget *register_frames_dialog ;
   GtkWidget *model_frames_dialog ;
   GtkWidget *match_stars_dialog ;
   GtkWidget *save_dialog ;
   CxcRes    *residuals ;

   /* Global settings */

   /* Pointers to menu item factory entries */
   GtkWidget  *frame_radio[6] ;
   GtkWidget  *frame_prev ;
   GtkWidget  *frame_next ;
   GtkWidget  *swap[2] ;
   GtkWidget  *align_wcs ;
   GtkWidget  *zoom_center ;
   GtkWidget  *zoom_radio1[9] ;
   GtkWidget  *zoom_radio2[5] ;
   GtkWidget  *scale_radio[4] ;
   GtkWidget  *analisys[5] ;
   GtkWidget  *match[3] ;
   GtkWidget  *pref_wcs_radio[2] ;
   GtkWidget  *pref_tunits_radio[2] ;
   GtkWidget  *pref_magnifier_radio[5] ;
   GtkWidget  *pref_tile_radio[3] ;
   GtkWidget  *pref_blink_radio[7] ;

} cxc_display ;

/*----------------------------------------------------------------------------
 * 							Function prototypes
 *--------------------------------------------------------------------------*/

extern void        cxc_display_init           (cxc_display   *display) ;
extern void        cxc_display_data           (cxc_display   *display) ;
extern void        cxc_display_map            (cxc_display   *display) ;
extern void        cxc_display_new            (cxc_display   *display) ;
extern void        cxc_main_window            (cxc_display   *display) ;

extern void        cxc_file_new               (GtkWidget     *widget, 
                                               cxc_display   *display) ;
extern void        cxc_file_close             (GtkWidget     *widget, 
                                               cxc_display   *display) ;

extern void        quit_program               (GtkWidget     *window,
                                               cxc_display   *display) ;

extern void        zoom_in                    (GtkWidget     *widget,
                                               cxc_display   *display) ;
extern void        zoom_out                   (GtkWidget     *widget,
                                               cxc_display   *display) ;
extern void        zoom_center                (GtkWidget     *widget,
                                               cxc_display   *display) ;
extern void        swap_x                     (GtkWidget     *widget,
                                               cxc_display   *display) ;
extern void        swap_y                     (GtkWidget     *widget,
                                               cxc_display   *display) ;

extern void        align_wcs                  (GtkWidget     *widget,
                                               cxc_display   *display) ;
extern void        align_maps                 (CxcMap        *current,
                                               CxcMap        *map) ;

extern void        cxc_buttons_wcs_status     (cxc_display   *display) ;
extern void        cxc_set_reference          (GtkWidget     *box,
                                               gboolean       expand,
                                               guint          border,
                                               gint           select,
                                               cxc_display   *display) ;

extern void        cxc_res_frame              (GtkWidget     *window,
                                               cxc_display   *display) ;

#endif /* !CXC_GTK_MAIN_H */
