/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 					      Bologna Astronomical Observatory
 *----------------------------------------------------------------------------
 * File name 	: cxc_init.c
 * Author 	: Paolo Montegriffo
 * Created on	: May, 2002	
 * Language	: ANSI C
 * Description	: Initialize GTK interface for CataXcorr
 * Version      : 2.1.14 - May 05, 2003
 *--------------------------------------------------------------------------*/

#if HAVE_CONFIG_H
#  include "config.h"
#endif

#include "common.h"

#include <gtk/gtk.h>

#include "cataxcorr.h"
#include "fileinfo.h"
#include "catalib.h"
#include "starlist.h"
#include "colormaps.h"
#include "cxc_decode.h"
#include "cpk_env.h"
#include "cpk_init.h"
#include "cxc_init.h"
#include "cxc_gtk_main.h"
#include "cxc_frames.h"

/*----------------------------------------------------------------------------
 * 							Function prototypes
 *--------------------------------------------------------------------------*/

static void      cxc_outinfo                          (cxc_display *display) ;

void cxc_message (int code, int v_level, const char *fmt, ...) ;

/*----------------------------------------------------------------------------
 * 							Global variables	
 *--------------------------------------------------------------------------*/

extern int        install_cmap ;
extern int        verbose ;
extern char      *module ;
extern boolean    no_interface ;

cxc_display *dsp ;
gint         wait_me ;

/*----------------------------------------------------------------------------
 *							Functions code
 *--------------------------------------------------------------------------*/

void
cxc_gtk_init (int ac, char **av)
{
   static cxc_display display ;
   dsp = &display ;

   /* Initialize Gtk application */
   gtk_init (&ac, &av) ;

   /* if (install_cmap) {
      gdk_rgb_init() ; 
      gtk_widget_set_default_colormap (gdk_rgb_get_cmap ()) ;
      gtk_widget_set_default_visual (gdk_rgb_get_visual ()) ;
      gdk_rgb_set_min_colors (min_colors) ;
   } */
   cpk_init ("cxc_logo.ppm") ;
   
   /* Decode list arguments */
   app_init_update_status ("Initializing Display struct", NULL, 0.00);
   cxc_display_init (&display) ;

   app_init_update_status ("Looking for data files", NULL, 0.10);
   cxc_arg_list_decode (ac, av) ;
   /* app_init_update_status (NULL, "Reading file ...", 0.05); *****/

   app_init_update_status ("Reading data files", NULL, 0.30);
   cxc_display_data (&display) ;
  
   app_init_update_status ("Initializing Maps", " ", 0.70);
   cxc_display_map (&display) ;

   app_init_update_status ("Initializing Frames", " ", 0.80);
   cxc_frames_init (&display) ;

   app_init_update_status ("Building Main Window", NULL, 1.00);
   cxc_display_new (&display) ;
   cxc_main_window (&display) ;
   cxc_outinfo (&display) ;
   wait_me = 0 ;

  /*  Things to do only if there is an interface  */
  if (no_interface == FALSE)
    {

    }

   /* Control */
   gtk_main () ;
}

#if HAVE_STDARG_H
#  include <stdarg.h>
#  define VA_START(a, f)        va_start(a, f)
#else
#  if HAVE_VARARGS_H
#    include <varargs.h>
#    define VA_START(a, f)      va_start(a)
#  endif
#endif
#ifndef VA_START
#   /* error no variadic api */
#endif

void
#if HAVE_STDARG_H
cxc_message (int code, int v_level, const char *fmt, ...)
#else
cxc_message (code, v_level, fmt, va_alist)
     int code ;
     int v_level ;
     const char *fmt ;
     va_dcl
#endif
{
   char   *msg ;
   char    ffmt[10];
   char    buffer[MAXPATHLEN] ;
   static  guint txt_pos = 0 ;
   va_list ap ;

   if(!verbose)
      return ;
   if (v_level > verbose)
      return ;

   if (no_interface == TRUE) {
      (void) sprintf (ffmt,"%%%ds",(int)strlen(module)+2) ;
      if(code) (void) fprintf (stdout, "%s: ",module) ;
      else     (void) fprintf (stdout,ffmt," ") ;
   }

   msg = (char*) strdup (fmt) ;

   VA_START (ap, fmt) ;

   (void) vsprintf (buffer, msg, ap) ;
   va_end (ap) ;
   free (msg) ;

   (void) fflush (stdout) ;
   if (no_interface == TRUE) {
      (void) fprintf (stdout, buffer) ;
      (void) fflush (stdout) ;
   }
   else {
      if (dsp->window == NULL) {
         strip_newline (buffer) ;
         app_init_update_status (NULL, buffer, 0.30) ;
      }
      else {
         GdkFont *fixed_font ;
         char     indent[81] ;

         /* if (v_level < 0) {
            if (!txt_pos)
               txt_pos = gtk_text_get_point (GTK_TEXT (dsp->console)) ;
            else
               gtk_text_set_point (GTK_TEXT (dsp->console), txt_pos) ;
         } */
         (void) sprintf (indent, "%*c", MIN (code*8, 80), 0x20) ;
         gtk_text_insert (GTK_TEXT (dsp->console), NULL, 
               &dsp->console->style->black, NULL, indent, -1) ;
         /* strip_newline (buffer) ; */
         /* fixed_font = gdk_font_load ("-misc-fixed-medium-r-*-*-*-140-*-*-*-*-*-*") ; */
         gtk_text_insert (GTK_TEXT (dsp->console), NULL, 
               &dsp->console->style->black, NULL, buffer, -1) ;
         /* gtk_text_insert (GTK_TEXT (dsp->console), NULL,
               &dsp->console->style->black, NULL, "\n", -1) ; */
         /* gdk_font_unref (fixed_font) ; */
      }
   }
}

static void 
cxc_outinfo (cxc_display *display)
{
   CxcData *data = CXC_DATA (display->data) ;
   Finfo  **f_info = data->f_set->info ;
   int      i ;
   char     line[MAX_RECORD_LENGTH] = "\n" ;
   double   ddd ;

   CXC_MESSAGE (1, 1, "*** File Info ***\n\n") ;
   for (i = 0; i < data->f_set->n; i++) {
      CXC_MESSAGE (0, 1, "File %s: %8d header lines, %8d data lines, %8d columns;\n",
      f_info[i]->filename, f_info[i]->n_head, f_info[i]->n_rows, f_info[i]->n_cols) ;  
      CXC_MESSAGE (2, 1, "%s Coordinates on col  %d,%d; reference mag on col  %d\n", 
      (f_info[i]->spheric ? "Spherical" : "Linear"),
      f_info[i]->x_pos+1, f_info[i]->y_pos+1, f_info[i]->m_pos+1) ;
      ddd = data->list[i]->inv_density ;
      CXC_MESSAGE (2, 1, "Mean star separation: %g [%s]\n",
            (f_info[i]->spheric ? ((ddd < 0.5) ? ddd*3600 : ddd  ) : ddd),
            (f_info[i]->spheric ? ((ddd < 0.5) ? "arcsec" : "deg") : "pixel")) ;
   } 

   strcpy (line," ") ;
   sprintf (line, "%g", data->gparam->NullValues[0]) ;
   for (i = 1; i < data->gparam->N_NullVal; i++) 
      sprintf (line, "%s,%g", line, data->gparam->NullValues[i]) ;
   CXC_MESSAGE (0,1,"Input Null Values: %s\n",line) ;

}
